@@ -37,20 +37,16 @@ class WebPDFExporter(HTMLExporter):
         """
     ).tag(config=True)
 
-    output_mimetype = "application/pdf"
+    output_mimetype = "text/html"
 
     @default('file_extension')
     def _file_extension_default(self):
-        return '.pdf'
+        return '.html'
 
     @default('template_name')
     def _template_name_default(self):
         return 'webpdf'
 
-    @default('template_data_paths')
-    def _template_data_paths_default(self):
-        return jupyter_path("nbconvert", "templates", "webpdf")
-
     disable_sandbox = Bool(
         False,
         help="""
@@ -91,11 +87,12 @@ class WebPDFExporter(HTMLExporter):
                 args=args
             )
             page = await browser.newPage()
+            await page.emulateMedia('screen')
             await page.waitFor(100)
             await page.goto(f'file://{temp_file.name}', waitUntil='networkidle0')
             await page.waitFor(100)
 
-            pdf_params = {}
+            pdf_params = {'printBackground': True}
             if not self.paginate:
                 # Floating point precision errors cause the printed
                 # PDF from spilling over a new page by a pixel fraction.
@@ -111,11 +108,10 @@ class WebPDFExporter(HTMLExporter):
                 width = dimensions['width']
                 height = dimensions['height']
                 # 200 inches is the maximum size for Adobe Acrobat Reader.
-                pdf_params = {
+                pdf_params.update({
                     'width': min(width, 200 * 72),
                     'height': min(height, 200 * 72),
-                    'printBackground': True,
-                }
+                })
             pdf_data = await page.pdf(pdf_params)
 
             await browser.close()
@@ -126,7 +122,7 @@ class WebPDFExporter(HTMLExporter):
         # Unfortunately, tempfile on Windows does not allow for an already open
         # file to be opened by a separate process. So we must close it first
         # before calling Chromium. We also specify delete=False to ensure the
-        # file file is not deleted after closing (the default behavior).
+        # file is not deleted after closing (the default behavior).
         temp_file = tempfile.NamedTemporaryFile(suffix=".html", delete=False)
         with temp_file:
             temp_file.write(html.encode('utf-8'))
@@ -158,3 +154,4 @@ class WebPDFExporter(HTMLExporter):
         resources['output_extension'] = '.pdf'
 
         return pdf_data, resources
+
